function [ data ] = collect( p_nemo_float, p_section, p_member, p_options )
%COLLECT Collects given section and/or member of float for every profile.
%   Iterates through every profile in the given nemo float and stores the
%   selected data either into a cell or a matrix depending on the class
%   that is being collected. This function is helpful if you require one
%   parameter of the float of all profiles in one variable.
%
%   Parameters:
%   p_nemo_float:        Float from which the data is being collected.
%   p_section:           String, Section identifier of the data that should
%                        be stored.
%   p_member:            String, member of the section that should be
%                        stored. If this equals an empty string, the
%                        complete section is being stored.
%   p_options:           Currently the following options can be passed to
%                        the function:
%                        
%                        'col2mat, {COLUMN_INTEGER}'
%                        -> collects the given column and combines it to a
%                        matrix (used for collecting pressure data of all
%                        profiles for example)
%
%                        'combine'
%                        -> collects the given section/member and combines
%                        it as a matrix
%
%   Returns:
%   data:                Collected data for every profile of the given
%                        float.

%% Initialize return variable
data = false;

%% Parameter check
parameter_error = false;

if (~iscell(p_nemo_float) || ~ischar(p_section))
    parameter_error = true;
end

if (exist('p_options', 'var') && ~ischar(p_options))
    parameter_error = true;
elseif (~exist('p_options', 'var'))
    p_options = '';
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Collect data

profile_count = length(p_nemo_float);

if (profile_count == 0)
    return;
end

% Calculate internal clock if option is given
if (strcmp(p_section, 'xmitDatevec'))
    data = collect_and_calculate_xmit_datevec(p_member);
    return
end

if ~exist('p_member', 'var') || strcmp(p_member, '') % the whole section is being collected
    
    switch class(p_nemo_float{1}.(p_section))
        
        case 'double' % it may be possible to store it as matrix
            % be sure that data is always one line
            one_line_double = true;
            column_count = 0;
            for o_profiles = 1:profile_count
                if ~isempty(p_nemo_float{o_profiles}.(p_section)) ...
                        && size(p_nemo_float{o_profiles}.(p_section), 1) ~= 1
                    one_line_double = false;
                    break;
                end
                if column_count == 0 && ~isempty(p_nemo_float{o_profiles}.(p_section))
                    column_count = size(p_nemo_float{o_profiles}.(p_section), 2);
                end
            end
            
            if one_line_double
                % only one line per profile, so create a matrix
                data = [];
                for o_profiles = 1:profile_count
                    if ~isempty(p_nemo_float{o_profiles}.(p_section))
                        data(o_profiles, :) = p_nemo_float{o_profiles}.(p_section);
                    else
                        data(o_profiles, 1:column_count) = NaN;
                    end
                end
            elseif (is_option_set('col2mat'))
                % the given column will be collected from all profiles and
                % combined into a column-wise matrix
                all_options = get_options(p_options);
                [column_number, status] = str2int(all_options{2});
                if (~status)
                    disp([mfilename ': Cannot convert option number 2 to integer!']);
                    return
                end
                data = [];
                for o_profiles = 1:profile_count
                    if isfield(p_nemo_float{o_profiles}, p_section)
                        data = [data, p_nemo_float{o_profiles}.(p_section)(:, column_number)];
                    end
                end
            elseif (is_option_set('combine'))
                % user selected to get a matrix containing all values
                % instead of seperated cells
                data = [];
                for o_profiles = 1:profile_count
                    if isfield(p_nemo_float{o_profiles}, p_section)
                        data = [data; p_nemo_float{o_profiles}.(p_section)];
                    end
                end
            else
                % create a struct because there are more lines in every profile
                data = {};
                for o_profiles = 1:profile_count
                    if ~isfield(p_nemo_float{o_profiles}, p_section)
                        data{o_profiles} = NaN;
                    else
                        data{o_profiles} = p_nemo_float{o_profiles}.(p_section);
                    end
                end
            end
            
        otherwise
            data = {};
            for o_profiles = 1:profile_count
                data{o_profiles} = p_nemo_float{o_profiles}.(p_section);
            end
            
    end
else
    % exact the same passage as above, but in this one there is a member given
    switch class(p_nemo_float{1}.(p_section).(p_member))
        
        case 'double'
            if size(p_nemo_float{1}.(p_section).(p_member), 1) == 1
                % only one line, so create a matrix
                data = [];
                for o_profiles = 1:profile_count
                    data(o_profiles, :) = p_nemo_float{o_profiles}.(p_section).(p_member);
                end
            else
                data = {};
                for o_profiles = 1:profile_count
                    data{o_profiles} = p_nemo_float{o_profiles}.(p_section).(p_member);
                end
            end
            
        otherwise
            data = {};
            for o_profiles = 1:profile_count
                data{o_profiles} = p_nemo_float{o_profiles}.(p_section).(p_member);
            end
    end
    
end

    function boolean = is_option_set(option_string)
        options = strsplit(p_options, ',');
        boolean = any(cellfun(@(x) strcmp(x, option_string), options));
    end

    function options = get_options(option_string)
        options = strtrim(strsplit(p_options, ','));
    end

    function data = collect_and_calculate_xmit_datevec(p_xmit_parameter_name)
        data = NaN(profile_count, 6);
        for i = 1:profile_count
            data(i, :) = ...
                datevec( ...
                    datetime([ ...
                              fliplr(p_nemo_float{i}.OVERALL_MISSION_INFORMATION.start_date), ...
                              p_nemo_float{i}.OVERALL_MISSION_INFORMATION.start_time 0 ...
                             ]) ...
                    + seconds(p_nemo_float{i}.PROFILE_TECHNICAL_DATA.(p_xmit_parameter_name)) ...
                );
        end
    end


end

